package com.energy.manage.service.service.analysis.impl;

import com.energy.manage.common.po.analysis.AnalysisOptionRecords;
import com.energy.manage.common.reponse.ConstVar;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.common.util.HttpUtils;
import com.energy.manage.service.constant.analysis.AnalysisConstants;
import com.energy.manage.service.domain.dto.analysis.AnalysisDto;
import com.energy.manage.service.domain.dto.analysis.AnalysisResultDto;
import com.energy.manage.service.domain.vo.analysis.*;
import com.energy.manage.service.domain.vo.windenginegroup.WindEngineGroupVo;
import com.energy.manage.service.domain.vo.windfieldbatch.WindFieldBatchVo;
import com.energy.manage.service.filter.LoginUser;
import com.energy.manage.service.filter.LoginUserContextHolder;
import com.energy.manage.service.mappers.analysis.*;
import com.energy.manage.service.mappers.windenginegroup.WindEngineGroupMapper;
import com.energy.manage.service.mappers.windfieldbatch.WindFieldBatchMapper;
import com.energy.manage.service.property.analysis.AlgorithmProperties;
import com.energy.manage.service.service.analysis.AnalysisService;
import com.energy.manage.service.util.BeanMapUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 分析服务
 * @author chy
 * @date 2024/5/23 12:59
 * @desc
 */
@Slf4j
@Service
public class AnalysisServiceImpl implements AnalysisService {


    @Autowired
    private AnalysisResultMapper analysisResultMapper;

    @Autowired
    private AnalysisTypeMapper analysisTypeMapper;

    @Autowired
    private WindEngineGroupMapper engineGroupMapper;

    @Autowired
    private AnalysisCommentDescriptionMapper commentDescriptionMapper;

    @Autowired
    private AnalysisCommentTypeRelationMapper commentTypeRelationMapper;

    @Autowired
    private AnalysisDiagramRelationMapper diagramRelationMapper;

    @Autowired
    private AnalysisGeneralFileMapper generalFileMapper;

    @Autowired
    private WindEngineGroupMapper windEngineGroupMapper;


    private static AnalysisOptionRecordsMapper optionRecordsMapper;


    private static WindFieldBatchMapper windFieldBatchMapper;

    /**
     * 操作内容
     */
    private static final String OPTION_CONTENT = "手动分析";

    private static ObjectMapper mapper = new ObjectMapper();

    /**
     * 分析结果列表
     * @param analysisResultDto
     * @return
     */
    @Override
    public List<AnalysisResultVo> analysisResultList(AnalysisResultDto analysisResultDto) {
        return analysisResultMapper.selectByCondition(analysisResultDto);
    }

    /**
     * 分析
     * @param analysisDto
     * @return
     */
    @Override
    public ResultResp analysis(AnalysisDto analysisDto) {
        try {
            if(checkAnalysising()){
                return ResultResp.FAIL(ConstVar.ANALYSISING);
            }
            //手动分析要进行分析类型code转换为flag（便于json分析类型赋值）
            // （是因为算法分析类型code与平台分析类型code没有做统一，开发之后才做统一，所以需要转换）
            if(analysisDto.getAutoOrManual().equalsIgnoreCase(AnalysisConstants.MANUAL_ANALYSIS)){
                transferAnalysisFlagByCode(analysisDto);
            }
            //将配置文件中的算法json转换为对象
            JsonNode rootNode = mapper.readTree(AlgorithmProperties.analysisJson);
            //将入参analysisDto转换为算法标准json
            dataProccess(analysisDto,rootNode);
            log.info("调用分析算法之前，将入参analysisDto转换为算法标准json = {}" , mapper.writeValueAsString(rootNode));
            System.out.println("json入参：" + rootNode.toString());
            //调用算法
            String result = HttpUtils.doPost(AlgorithmProperties.analysisUrl,mapper.writeValueAsString(rootNode));
            log.info("调用算法分析返回结果：{}",result);
            //手动操作进行操作记录添加
            if(analysisDto.getAutoOrManual().equals(AnalysisConstants.MANUAL_ANALYSIS)){
                addOptionRecords(analysisDto.getPowerFarmID(),null,OPTION_CONTENT);
            }
            return ResultResp.SUCCESS();
        } catch (JsonProcessingException e) {
            log.error("批次号 = {} 手动分析异常",analysisDto.getDataBatchNum(),e);
            return ResultResp.FAIL();
        }
    }

    /**
     * 分析编辑查询（查询分析类型、风机组）
     * @param batchCode
     * @return
     */
    @Override
    public AnalysisResultShowVo analysisEditQuery(String batchCode){
        AnalysisResultShowVo analysisResultShowVo = new AnalysisResultShowVo();
        return analysisResultShowVo.setAnalysisTypeList(getAnalysisType())
                .setWindEngineGroupList(getWindEngineGroupByBatchCode(batchCode));
    }


    /**
     * 分析详情
     * @param batchCode
     * @param analysisTypeCode
     * @param fieldEngineCode
     * @return
     */
    @Override
    public List<AnalysisDetailVo> analysisDetail(String batchCode, String analysisTypeCode, String fieldEngineCode) {
        List<AnalysisDetailVo> analysisDetailVos = new ArrayList<>();
        //获取所有的分析类型
        List<AnalysisTypeVo> analysisTypeVos = analysisTypeMapper.selectByTypeCode(analysisTypeCode);
        if(CollectionUtils.isEmpty(analysisTypeVos)){
            return analysisDetailVos;
        }
        //最新总文件
        List<AnalysisGeneralFileVo> newGeneralFileVos = getNewGeneralFiles(batchCode,analysisTypeCode);
        //获取最新分文件
        List<AnalysisDiagramRelationVo> newDiagramRelationVos = getNewDiagramRelations(batchCode, analysisTypeCode, fieldEngineCode);
        //查询评论
        List<AnalysisCommentDescriptionVo> commentDescriptionVos = commentDescriptionMapper.selecByBatchCodeAndAnalysisTypeCode(batchCode,analysisTypeCode);
        //根据分析类型设置分析结果
        analysisTypeVos.forEach(analysisTypeVo -> {
            //设置相应参数
            AnalysisDetailVo analysisDetailVo = getAnalysisDetailVo(analysisTypeVo, newGeneralFileVos,newDiagramRelationVos,commentDescriptionVos);
            if(analysisDetailVo != null){
                analysisDetailVos.add(analysisDetailVo);
            }
        });
        //只有当分析类型不为空的时候才返回评论类型
        if(StringUtils.isNotBlank(analysisTypeCode) && CollectionUtils.isNotEmpty(analysisDetailVos)){
            analysisDetailVos.get(0).setCommentTypeRelations(commentTypeRelationMapper.selectByAnalysisTypeCode(batchCode,analysisTypeCode));
        }
        return analysisDetailVos;
    }


    /**
     * 分析中检查
     * @return
     */
    @Override
    public boolean checkAnalysising(){
        return analysisResultMapper.selectAlasisingCount() > AnalysisConstants.SERVER_MERCHINE_COUNT;
    }


    /**
     * 获取分析类型
     */
    private List<Map<String,Object>> getAnalysisType(){
        List<Map<String,Object>> resultList = new ArrayList<>();
        List<AnalysisTypeVo> analysisTypeVos =  analysisTypeMapper.selectAllVo();
        if(CollectionUtils.isEmpty(analysisTypeVos)){
            return resultList;
        }
        analysisTypeVos = analysisTypeVos.stream().filter(item->StringUtils.isNotBlank(item.getTypeCode())).sorted(Comparator.comparing(AnalysisTypeVo::getParentId)).collect(Collectors.toList());
        analysisTypeVos.forEach(analysisTypeVo -> resultList.add(BeanMapUtils.beanToMap(analysisTypeVo)));
        return resultList;
//        return TreeUtil.getJava8ResultTree(resultList, AnalysisConstants.ANALYSIS_TYPE_PARRENT_KEY,AnalysisConstants.ANALYSIS_TYPE_KEY);
    }

    /**
     * 获取风机组编号
     * @param batchCode
     * @return
     */
    private List<WindEngineGroupVo> getWindEngineGroupByBatchCode(String batchCode){
        return engineGroupMapper.selectWindEngineGroupByBatchCode(batchCode);
//        if(CollectionUtils.isEmpty(engineGroupVos)){
//            return new HashMap<>();
//        }
//        return engineGroupVos.parallelStream().collect(Collectors.toMap(WindEngineGroupVo::getEngineCode,WindEngineGroupVo::getEngineName, (k1, k2) -> k1, LinkedHashMap::new));
    }


    /**
     * 获取总文件
     * @param batchCode
     * @param analysisTypeCode
     * @return
     */
    private List<AnalysisGeneralFileVo> getNewGeneralFiles(String batchCode,String analysisTypeCode){
        List<AnalysisGeneralFileVo> generalFileVos = generalFileMapper.selectByCondition(batchCode,analysisTypeCode);
        generalFileVos.sort(Comparator.comparing(AnalysisGeneralFileVo::getCreateTime,Comparator.reverseOrder()));
        //删除地址相同的文件
        return generalFileVos.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(AnalysisGeneralFileVo::getFileAddr))),
                ArrayList :: new));
    }


    /**
     * 获取最新分析结果分文件
     * @param batchCode
     * @param analysisTypeCode
     * @param fieldEngineCode
     * @return
     */
    private List<AnalysisDiagramRelationVo> getNewDiagramRelations(String batchCode,String analysisTypeCode,String fieldEngineCode){
        List<AnalysisDiagramRelationVo> diagramRelationVos = diagramRelationMapper.selectByCondition(batchCode,analysisTypeCode,fieldEngineCode);
        if(CollectionUtils.isEmpty(diagramRelationVos)){
            return diagramRelationVos;
        }
        //如果风机编号不为空的话则不对风机编号进行排序
        if (StringUtils.isNotBlank(fieldEngineCode)) {
            diagramRelationVos.sort(Comparator.comparing(AnalysisDiagramRelationVo::getCreateTime, Comparator.reverseOrder()));
        } else {
            diagramRelationVos.sort(Comparator.comparing(AnalysisDiagramRelationVo::getFieldEngineCode)
                    .thenComparing(AnalysisDiagramRelationVo::getCreateTime, Comparator.reverseOrder()));
        }
        //去除地址相同的对象
        return diagramRelationVos.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(AnalysisDiagramRelationVo::getFileAddr))),
                                                     ArrayList :: new)
        );
    }

    /**
     * 设置分析结果
     * @param analysisTypeVo
     * @param newGeneralFileVos
     * @param newDiagramRelationVos
     * @param commentDescriptionVos
     * @return
     */
    private AnalysisDetailVo getAnalysisDetailVo(AnalysisTypeVo analysisTypeVo,
                                                 List<AnalysisGeneralFileVo> newGeneralFileVos,
                                                 List<AnalysisDiagramRelationVo> newDiagramRelationVos,
                                                 List<AnalysisCommentDescriptionVo> commentDescriptionVos){
        List<AnalysisGeneralFileVo> generalFileVos = null;
        if(CollectionUtils.isNotEmpty(newGeneralFileVos)) {
            generalFileVos = newGeneralFileVos.stream().filter(analysisGeneralFileVo -> analysisGeneralFileVo.getAnalysisTypeCode().equals(analysisTypeVo.getTypeCode())).collect(Collectors.toList());
        }
        List<AnalysisDiagramRelationVo> diagramRelationVos = null;
        if(CollectionUtils.isNotEmpty(newDiagramRelationVos)){
            diagramRelationVos = newDiagramRelationVos.stream().filter(analysisDiagramRelationVo -> analysisDiagramRelationVo.getAnalysisTypeCode().equals(analysisTypeVo.getTypeCode())).collect(Collectors.toList());
        }
        if(CollectionUtils.isEmpty(generalFileVos) && CollectionUtils.isEmpty(diagramRelationVos)){
            return null;
        }
        return new AnalysisDetailVo()
                .setAnalysisTypeCode(analysisTypeVo.getTypeCode())
                .setAnalysisTypeName(analysisTypeVo.getTypeName())
                .setGeneralFiles(generalFileVos)
                .setDiagramRelations(diagramRelationVos)
                .setCommentDescriptionVos(commentDescriptionVos);
    }

    /**
     * 根据分析类型code转换分析flag
     * @param analysisDto
     */
    private void transferAnalysisFlagByCode(AnalysisDto analysisDto){
        List<String> configAnalysisCodes = analysisDto.getConfigAnalysis();
        if(CollectionUtils.isEmpty(configAnalysisCodes)){
            return;
        }
        analysisDto.setConfigAnalysis(analysisTypeMapper.selecFlagtByCode(configAnalysisCodes));
    }

    /**
     * 数据处理
     * @param analysisDto 前端入参
     * @param rootNode 根节点json
     */
    private void dataProccess(AnalysisDto analysisDto,JsonNode rootNode){
        //前端入参类
        Field[] analysisFields = analysisDto.getClass().getDeclaredFields();
        String scada = analysisDto.getScada();
        Arrays.asList(analysisFields).stream().forEach(field -> {
            field.setAccessible(true);
            try {
                //值不能为空且值为list的时候list长度不能为0
                if(field.get(analysisDto) != null){
                    updateNodesByPropertyName(rootNode, field.getName(), field.get(analysisDto),scada);
                }
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        });
    }

    /**
     * 测试排序去重
     * @param args
     * @throws InterruptedException
     */
    public static void main(String[] args) throws InterruptedException {
        List<AnalysisGeneralFileVo> generalFileVos = new ArrayList<>();
        AnalysisGeneralFileVo a1 = new AnalysisGeneralFileVo();
        a1.setFileAddr("http://123/412/89.html");
        a1.setCreateTime(new Date());
        a1.setBatchCode("123");
        generalFileVos.add(a1);
        Thread.sleep(1000);
        AnalysisGeneralFileVo a2 = new AnalysisGeneralFileVo();
        a2.setFileAddr("http://123/412/89.html");
        a2.setCreateTime(new Date());
        generalFileVos.add(a2);
        a2.setBatchCode("453");
        generalFileVos.sort(Comparator.comparing(AnalysisGeneralFileVo::getCreateTime,Comparator.reverseOrder()));
        //删除地址相同的文件
        List<AnalysisGeneralFileVo> returnList =  generalFileVos.stream().collect(Collectors.collectingAndThen(
                                                    Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(AnalysisGeneralFileVo::getFileAddr))),
                                                    ArrayList :: new));
        returnList.forEach(System.out::println);
    }

    /**
     * 根据属性修改json值
     * @param parentNode 节点
     * @param propertyName 属性名称
     * @param newValue 新值
     */
    private void updateNodesByPropertyName(JsonNode parentNode, String propertyName, Object newValue,String scada) {
        //遍历当前节点
        for (Iterator<Map.Entry<String, JsonNode>> it = parentNode.fields(); it.hasNext(); ) {
            Map.Entry<String, JsonNode> entry = it.next();
            // 匹配到属性名称，进行值的修改
            if (entry.getKey().equalsIgnoreCase(propertyName)) {
                //判断当前节点有没有子节点 排除分析节点
                if(!entry.getValue().fields().hasNext() && !propertyName.equalsIgnoreCase(AlgorithmProperties.configAnalysisProperties)){
                    //数组属性得特殊处理
                    if(AlgorithmProperties.arrayProperties.contains(propertyName)){
                        ArrayNode arrayNode = mapper.createArrayNode();
                        ((List<String>)newValue).stream().forEach(item-> arrayNode.add(item));
                        ((ObjectNode) parentNode).set(propertyName,arrayNode);
                    }else{
                        ((ObjectNode) parentNode).put(propertyName, newValue.toString());
                    }
                }else{
                    assignmentByPropertyName(entry.getValue(), propertyName, newValue,scada);
                }
                return;
            }
            // 递归搜索子节点
            updateNodesByPropertyName(entry.getValue(), propertyName, newValue,scada);
        }

    }

    /**
     * 处理特殊属性
     * @param subNode
     * @param propertyName
     * @param newValue
     */
    private void assignmentByPropertyName(JsonNode subNode, String propertyName, Object newValue,String scada){
        //scada数据筛选过滤 、图像设置特殊处理
        if(AlgorithmProperties.graphSetsProperties.contains(propertyName)
                || AlgorithmProperties.customFilterProperties.contains(propertyName)){
            dealGraphSetsAndCustomFilter(subNode, newValue);
            return ;
        }
        //分析类型特殊处理
        if(AlgorithmProperties.configAnalysisProperties.equals(propertyName)){
            dealConfigAnalysis(subNode, newValue,scada);
            return;
        }
    }

    /**
     * scada数据筛选过滤 、图像设置特殊处理
     * @param subNode
     * @param newValue
     */
    private void dealGraphSetsAndCustomFilter(JsonNode subNode, Object newValue){
        List<Integer> list = (List) newValue;
        if(list.size() == 0){
            return;
        }
        AtomicInteger index = new AtomicInteger(0);
        subNode.fields().forEachRemaining(entry -> {
            ((ObjectNode) subNode).put(entry.getKey(), list.get(index.get()));
            index.incrementAndGet();
        });
    }

    /**
     * 分析类型处理
     * @param subNode
     * @param newValue
     */
    private void dealConfigAnalysis(JsonNode subNode, Object newValue,String scada){
        try{
            ArrayNode arrayNode = (ArrayNode) subNode;
            List<String> analysisTypes = (List) newValue;
            for(int i = 0 ; i < analysisTypes.size() ; i++){
                JsonNode childNode = mapper.readTree(AlgorithmProperties.configAnalysisJson);
                //分析类型解析
                String[] types = analysisTypes.get(i).split("\\|");
                AtomicInteger index =  new AtomicInteger(0);
                childNode.fields().forEachRemaining(entry-> {
                    if(StringUtils.isNotBlank(scada) && entry.getKey().equals(AnalysisConstants.SCADA)){
                        ((ObjectNode)childNode).put(entry.getKey(), scada);
                    }else{
                        ((ObjectNode)childNode).put(entry.getKey(), types[index.get()]);
                    }
                    index.incrementAndGet();
                });
                arrayNode.add(childNode);
            }
        }catch (JsonProcessingException jsonExcep){
            log.error("分析类型解析失败：",jsonExcep);
        }
    }

    /**
     * 操作记录
     * @param fieldCode
     */
    public static void addOptionRecords(String fieldCode,String batchCode,String optionContent){
        try{
            if(StringUtils.isBlank(fieldCode)){
                WindFieldBatchVo windFieldBatchVo = windFieldBatchMapper.selectByBatchCode(batchCode);
                fieldCode = windFieldBatchVo.getFieldCode();
            }
            AnalysisOptionRecords optionRecordsPo = new AnalysisOptionRecords();
            LoginUser user = LoginUserContextHolder.get();
            optionRecordsPo.setOptionBy(user.getUserId());
            optionRecordsPo.setOptionByName(user.getUserName());
            optionRecordsPo.setOptionContent(optionContent);
            optionRecordsPo.setFieldCode(fieldCode);
            optionRecordsMapper.insertSelective(optionRecordsPo);
        }catch (Exception e){
            log.info("增加操作记录异常",e);
            return;
        }
    }

    /**
     * 获取 ratedCapacity 风机额定功率*1.2
     * @param analysisDto
     * @return
     */
    private String getRatedCapacity(AnalysisDto analysisDto){
        //获取风机机组
        List<String> engineCodes = analysisDto.getTurbines();
        //获取风场编号
        String fieldCode = analysisDto.getPowerFarmID();
        //查询最大功率
       BigDecimal ratedCapacity = windEngineGroupMapper.selectMaxRatedCapacity(fieldCode,engineCodes);
       //功率因数 1.2
       BigDecimal powerFactorRated = new BigDecimal(AlgorithmProperties.powerFactorRated);
       //返回功率
       return ratedCapacity == null ? "0" : ratedCapacity.multiply(powerFactorRated).toString();
    }

    @Autowired
    private void setAnalysisOptionRecordsMapper(AnalysisOptionRecordsMapper optionRecordsMapper) {
        AnalysisServiceImpl.optionRecordsMapper = optionRecordsMapper;
    }

    @Autowired
    private void setWindFieldBatchMapper(WindFieldBatchMapper windFieldBatchMapper) {
        AnalysisServiceImpl.windFieldBatchMapper = windFieldBatchMapper;
    }

}
